/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.model.fixings;

import java.util.List;

import org.dive4elements.river.artifacts.math.fitting.LogLinearAlternative;
import org.dive4elements.river.artifacts.model.fixings.fitting.LinearLogLinearizedFitting;
import org.dive4elements.river.artifacts.model.fixings.fitting.LinearLogLinearizedFitting.Result;

/**
 * @author Gernot Belger
 */
final class LogLinearFittingOperation extends AbstractFittingOperation implements IFittingOperation {

    public LogLinearFittingOperation(final LogLinearAlternative function) {
        super(function);
    }

    @Override
    public Fitting execute(final List<FittingData> data) {

        final double[] obsDischarges = new double[data.size()];
        final double[] obsWaterlevels = new double[data.size()];

        for (int i = 0; i < data.size(); i++) {
            final FittingData fittingData = data.get(i);

            obsDischarges[i] = fittingData.q;
            obsWaterlevels[i] = fittingData.w;
        }

        final LinearLogLinearizedFitting fitting = new LinearLogLinearizedFitting(obsDischarges, obsWaterlevels);

        final Result result = fitting.optimize();
        final double a = result.getA();
        final double b = result.getB();
        final double m = result.getM();
        final double chiSquare = result.getChiSquare();

        final double[] parameters = new double[] { a, m, b };

        return createFitting(parameters, chiSquare, data);
    }
}